﻿using Syncfusion.SfPdfViewer.XForms;
using System;
using System.IO;
using System.Threading.Tasks;

using Xamarin.Forms;
using Xamarin.Forms.Xaml;

namespace BImprendo.Pages.PDF
{
    [XamlCompilation(XamlCompilationOptions.Compile)]
    public partial class PdfPage : ContentPage
    {
        public event EventHandler<Boolean> OperationDone;
        Stream currentStream = null;
        string currentFilename = null;
        bool MODEpdfView = false;

        SfPdfViewer pdfView = null;

        public PdfPage(string title = "")
        {
            this.Title = title;
            InitializeComponent();
            //NavigationPage.SetHasBackButton(this, false);

            loader.IsRunning = true;
            loader.IsVisible = true;

            //BACK
            ToolbarItem tbBack = new ToolbarItem();
            tbBack.Text = "Indietro";
            tbBack.Icon = "Assets/ICOback.png";
            tbBack.Order = ToolbarItemOrder.Primary;
            tbBack.Clicked += (sender, args) => { OperationDone?.Invoke(this, false); };
            if (Device.Idiom == TargetIdiom.Desktop || Device.Idiom == TargetIdiom.Tablet)
                ToolbarItems.Add(tbBack);

            //SHARE
            ToolbarItem tbShare = new ToolbarItem();
            tbShare.Text = "Condividi";
            //tbShare.Icon = Manager.Instance.GetImageString("ICOShare.png");
            tbShare.Order = ToolbarItemOrder.Primary;
            tbShare.Clicked += async (sender, args) => {
                try
                {
                    Share();
                }
                catch (Exception ex)
                {
                    //await Manager.Instance.MessageError(this, "Errore durante la condivisione del file");
                }
            };

            //SAVE
            ToolbarItem tbSave = new ToolbarItem();
            tbSave.Text = "Salva";
            //tbSave.Icon = Manager.Instance.GetImageString("ICOSave.png");
            tbSave.Order = ToolbarItemOrder.Primary;
            tbSave.Clicked += async (sender, args) => {
                try
                {
                    Save();
                }
                catch (Exception ex)
                {
                    //await Manager.Instance.MessageError(this, ex.Message, "Errore il salvataggio del file");
                }
            };


            //OPEN
            ToolbarItem tbOpenExt = new ToolbarItem();
            tbOpenExt.Text = "Apri";
            //tbOpenExt.Icon = Manager.Instance.GetImageString("ICODocument.png");
            tbOpenExt.Order = ToolbarItemOrder.Primary;
            tbOpenExt.Clicked += async (sender, args) => {
                try
                {
                    OpenExternal();
                }
                catch (Exception ex)
                {
                    //await Manager.Instance.MessageError(this, ex.Message, "Errore il salvataggio del file");
                }
            };

            this.ToolbarItems.Add(tbOpenExt);
            this.ToolbarItems.Add(tbSave);
            this.ToolbarItems.Add(tbShare);

            bool previewOK = PdfPage.IsPreviewOK();
            if (previewOK) BuildPdfView();
            else lblInfo.IsVisible = true;            
        }


        /// <summary>
        /// Se il Sistema supporta il PDFViewer
        /// </summary>
        /// <returns></returns>
        public static bool IsPreviewOK()
        {
            return true;
            /*
            string currentVersion = "NONE";
            if (CrossDeviceInfo.IsSupported)
                currentVersion = CrossDeviceInfo.Current.Version;
            int currentVersionInt = 0;
            Int32.TryParse(currentVersion.Substring(0, 1), out currentVersionInt);
            if (Device.RuntimePlatform == Device.UWP)
            {
                return true;
            }
            if (Device.RuntimePlatform == Device.Android && currentVersionInt >= 5)
            {
                return true;
            }
            if (Device.RuntimePlatform == Device.iOS && currentVersionInt >= 9)
            {
                return true;
            }
            return false;
            */
        }
       
        public async Task BuildGUI()
        {
            
        }

        public async Task LoadDocument(Stream s, string filename)
        {
            try
            {
                loader.IsRunning = true;
                loader.IsVisible = true;
                currentStream = s;
                currentFilename = filename;
               
                if (MODEpdfView) pdfView.LoadDocument(currentStream);
                else OpenExternal();                
            }
            catch(Exception)
            {
                //await Manager.Instance.MessageError(this, "Impossibile visualizzare il documento");
            }
            finally
            {
                loader.IsRunning = false;
                loader.IsVisible = false;
            }
        }


        private void BuildPdfView()
        {
            stackPDF.Children.Clear();
            if (pdfView == null)
            {
                pdfView = new SfPdfViewer
                {
                    HorizontalOptions = LayoutOptions.FillAndExpand,
                    VerticalOptions = LayoutOptions.FillAndExpand,
                    IsTextSelectionEnabled = false,
                    
                };

            }
            MODEpdfView = true;
            stackPDF.Children.Add(pdfView);
        }

        public void UnloadDocument()
        {
            if(MODEpdfView && pdfView!=null){
             try{
                    pdfView.Unload();
                }catch(Exception)
                {

                }
            }
            if (currentStream != null) currentStream.Close();
        }

        /*
        private FSitem CreateFSitem()
        {
            byte[] data = new byte[currentStream.Length];
            currentStream.Position = 0;
            currentStream.Read(data, 0, (int)currentStream.Length);
            currentStream.Position = 0;

            FSitem item = new FSitem()
            {
                Data = data,
                Filename = currentFilename
            };
            
            return item;
        }
        */
        private async void OpenExternal()
        {
            /*
            bool permission = await Manager.Instance.PermissionCheck(Plugin.Permissions.Abstractions.Permission.Storage);
            if (!permission) return;

            I_PdfView ds = DependencyService.Get<I_PdfView>();
            if (ds == null)
            {
                await Manager.Instance.MessageError(this, "Funzionalità non implementata sulla tua piattaforma");
                return;
            }

            FSitem item = CreateFSitem();            
            ds.OpenPdf(item);            
            item = null;
            */
        }

        private async void Share()
        {
            /*
            I_Share ds = DependencyService.Get<I_Share>();
            if (ds == null)
            {
                await Manager.Instance.MessageError(this, "Funzionalità non implementata sulla tua piattaforma");
                return;
            }

            FSitem item = CreateFSitem();
            await ds.ShareFile("Condivisione Documento", item.Description, item);
            item = null; 
            */
        }


        private async void Save()
        {
            /*
            I_FilesystemPicker ds = DependencyService.Get<I_FilesystemPicker>();
            if (ds == null)
            {
                await Manager.Instance.MessageError(this, "Funzionalità non implementata sulla tua piattaforma");
                return;
            }

            FSitem item = CreateFSitem();
            await ds.SelectDirAndSave(item.Data, item.Filename);
            item = null;  
            */
        }


    }
}
